@@ -14,7 +14,6 @@ from typing import NamedTuple
 
 import pytest
 
-from virtualenv.create.via_global_ref.builtin.cpython.common import is_macos_brew
 from virtualenv.discovery import cached_py_info
 from virtualenv.discovery.py_info import PythonInfo, VersionInfo
 from virtualenv.discovery.py_spec import PythonSpec
@@ -23,6 +22,7 @@ from virtualenv.info import IS_PYPY, IS_WIN, fs_supports_symlink
 CURRENT = PythonInfo.current_system()
 
 
+@pytest.mark.graalpy
 def test_current_as_json():
     result = CURRENT._to_json()  # noqa: SLF001
     parsed = json.loads(result)
@@ -49,12 +49,12 @@ def test_bad_exe_py_info_no_raise(tmp_path, caplog, capsys, session_app_data):
     out, _ = capsys.readouterr()
     assert not out
     messages = [r.message for r in caplog.records if r.name != "filelock"]
-    assert len(messages) == 2
-    msg = messages[0]
-    assert "get interpreter info via cmd: " in msg
-    msg = messages[1]
-    assert str(exe) in msg
-    assert "code" in msg
+    assert len(messages) == 4  # two subprocess attempts (debug cmd + retry debug) + final info
+    assert "get interpreter info via cmd: " in messages[0]
+    assert "retrying" in messages[1]
+    assert "get interpreter info via cmd: " in messages[2]
+    assert str(exe) in messages[3]
+    assert "code" in messages[3]
 
 
 @pytest.mark.parametrize(
@@ -129,7 +129,7 @@ def test_py_info_cached_error(mocker, tmp_path, session_app_data):
         PythonInfo.from_exe(str(tmp_path), session_app_data)
     with pytest.raises(RuntimeError):
         PythonInfo.from_exe(str(tmp_path), session_app_data)
-    assert spy.call_count == 1
+    assert spy.call_count == 2  # initial attempt + one retry, second from_exe uses cache
 
 
 @pytest.mark.skipif(not fs_supports_symlink(), reason="symlink is not supported")
@@ -141,18 +141,22 @@ def test_py_info_cached_symlink_error(mocker, tmp_path, session_app_data):
     symlinked.symlink_to(tmp_path)
     with pytest.raises(RuntimeError):
         PythonInfo.from_exe(str(symlinked), session_app_data)
-    assert spy.call_count == 2
+    assert spy.call_count == 4  # 2 attempts per path (initial + retry)
 
 
 def test_py_info_cache_clear(mocker, session_app_data):
-    spy = mocker.spy(cached_py_info, "_run_subprocess")
     result = PythonInfo.from_exe(sys.executable, session_app_data)
     assert result is not None
-    count = 1 if result.executable == sys.executable else 2  # at least two, one for the venv, one more for the host
-    assert spy.call_count >= count
+
     PythonInfo.clear_cache(session_app_data)
-    assert PythonInfo.from_exe(sys.executable, session_app_data) is not None
-    assert spy.call_count >= 2 * count
+    assert not cached_py_info._CACHE  # noqa: SLF001
+
+    spy = mocker.spy(cached_py_info, "_run_subprocess")
+    info = PythonInfo.from_exe(sys.executable, session_app_data)
+    assert info is not None
+
+    native_difference = 1 if info.system_executable == info.executable else 0
+    assert spy.call_count + native_difference >= 1
 
 
 def test_py_info_cache_invalidation_on_py_info_change(mocker, session_app_data):
@@ -162,7 +166,7 @@ def test_py_info_cache_invalidation_on_py_info_change(mocker, session_app_data):
     # 2. Spy on _run_subprocess
     spy = mocker.spy(cached_py_info, "_run_subprocess")
 
-    # 3. Modify the content of py_info.py
+    # 3. Backup py_info.py
     py_info_script = Path(cached_py_info.__file__).parent / "py_info.py"
     original_content = py_info_script.read_text(encoding="utf-8")
     original_stat = py_info_script.stat()
@@ -170,19 +174,19 @@ def test_py_info_cache_invalidation_on_py_info_change(mocker, session_app_data):
     try:
         # 4. Clear the in-memory cache
         mocker.patch.dict(cached_py_info._CACHE, {}, clear=True)  # noqa: SLF001
+
+        # 5. Modify py_info.py to invalidate the cache
         py_info_script.write_text(original_content + "\n# a comment", encoding="utf-8")
 
-        # 5. Get the PythonInfo object again
+        # 6. Get the PythonInfo object again
         info = PythonInfo.from_exe(sys.executable, session_app_data)
 
-        # 6. Assert that _run_subprocess was called again
-        if is_macos_brew(info):
-            assert spy.call_count in {2, 3}
-        else:
-            assert spy.call_count == 2
+        # 7. Assert that _run_subprocess was called again
+        native_difference = 1 if info.system_executable == info.executable else 0
+        assert spy.call_count + native_difference >= 2
 
     finally:
-        # Restore the original content and timestamp
+        # 8. Restore the original content and timestamp
         py_info_script.write_text(original_content, encoding="utf-8")
         os.utime(str(py_info_script), (original_stat.st_atime, original_stat.st_mtime))
 
@@ -371,10 +375,7 @@ def test_py_info_to_system_raises(session_app_data, mocker, caplog):
 
 
 def _stringify_schemes_dict(schemes_dict):
-    """
-    Since this file has from __future__ import unicode_literals, we manually cast all values of mocked install_schemes
-    to str() as the original schemes are not unicode on Python 2.
-    """
+    """Since this file has from __future__ import unicode_literals, we manually cast all values of mocked install_schemes to str() as the original schemes are not unicode on Python 2."""
     return {str(n): {str(k): str(v) for k, v in s.items()} for n, s in schemes_dict.items()}
 
 
@@ -433,7 +434,9 @@ def test_custom_venv_install_scheme_is_prefered(mocker):
     assert pyinfo.install_path("purelib").replace(os.sep, "/") == f"lib/python{pyver}/site-packages"
 
 
-@pytest.mark.skipif(not (os.name == "posix" and sys.version_info[:2] >= (3, 11)), reason="POSIX 3.11+ specific")
+@pytest.mark.skipif(
+    IS_PYPY or not (os.name == "posix" and sys.version_info[:2] >= (3, 11)), reason="POSIX 3.11+ specific"
+)
 def test_fallback_existent_system_executable(mocker):
     current = PythonInfo()
     # Posix may execute a "python" out of a venv but try to set the base_executable
@@ -524,3 +527,144 @@ def test_uses_posix_prefix_on_debian_3_10_without_venv(mocker):
     pyver = f"{pyinfo.version_info.major}.{pyinfo.version_info.minor}"
     assert pyinfo.install_path("scripts") == "bin"
     assert pyinfo.install_path("purelib").replace(os.sep, "/") == f"lib/python{pyver}/site-packages"
+
+
+def test_sysconfig_vars_include_shared_lib_keys() -> None:
+    for key in ("Py_ENABLE_SHARED", "INSTSONAME", "LIBDIR"):
+        assert key in CURRENT.sysconfig_vars
+
+
+def test_py_info_has_sysconfig_platform():
+    """PythonInfo should have a sysconfig_platform field from sysconfig.get_platform()."""
+    assert hasattr(CURRENT, "sysconfig_platform")
+    assert CURRENT.sysconfig_platform is not None
+    assert isinstance(CURRENT.sysconfig_platform, str)
+    assert len(CURRENT.sysconfig_platform) > 0
+
+
+def test_py_info_machine_property():
+    """machine property should return a non-empty ISA string derived from sysconfig_platform."""
+    machine = CURRENT.machine
+    assert machine is not None
+    assert isinstance(machine, str)
+    assert len(machine) > 0
+    # Should be a recognized ISA value
+    known_isas = {"arm64", "x86_64", "x86", "ppc64le", "ppc64", "s390x", "riscv64"}
+    assert machine in known_isas, f"unexpected machine value: {machine}"
+
+
+def test_py_info_machine_in_spec():
+    """The spec property should include the machine ISA."""
+    spec = CURRENT.spec
+    assert CURRENT.machine in spec
+    # Format: Implementation + version + optional 't' + '-' + arch + '-' + machine
+    assert f"-{CURRENT.architecture}-{CURRENT.machine}" in spec
+
+
+def test_py_info_sysconfig_platform_matches_sysconfig():
+    """sysconfig_platform field should match sysconfig.get_platform()."""
+    assert CURRENT.sysconfig_platform == sysconfig.get_platform()
+
+
+@pytest.mark.parametrize(
+    ("platform", "expected"),
+    [
+        pytest.param("win32", "x86", id="win32"),
+        pytest.param("win-amd64", "x86_64", id="win-amd64"),
+        pytest.param("win-arm64", "arm64", id="win-arm64"),
+        pytest.param("linux-x86_64", "x86_64", id="linux-x86_64"),
+        pytest.param("linux-aarch64", "arm64", id="linux-aarch64"),
+        pytest.param("linux-riscv64", "riscv64", id="linux-riscv64"),
+        pytest.param("linux-ppc64le", "ppc64le", id="linux-ppc64le"),
+        pytest.param("linux-s390x", "s390x", id="linux-s390x"),
+        pytest.param("macosx-14.0-arm64", "arm64", id="macos-arm64"),
+        pytest.param("macosx-14.0-x86_64", "x86_64", id="macos-x86_64"),
+    ],
+)
+def test_py_info_machine_derivation(platform, expected):
+    info = copy.deepcopy(CURRENT)
+    info.sysconfig_platform = platform
+    assert info.machine == expected
+
+
+@pytest.mark.parametrize("runtime_isa", ["arm64", "x86_64"])
+def test_py_info_machine_derivation_universal2(mocker, runtime_isa):
+    info = copy.deepcopy(CURRENT)
+    info.sysconfig_platform = "macosx-11.0-universal2"
+    mocker.patch("virtualenv.discovery.py_info.platform.machine", return_value=runtime_isa)
+    assert info.machine == runtime_isa
+
+
+def test_py_info_satisfies_with_machine():
+    """Current interpreter should satisfy a spec that includes its own machine."""
+    threaded = "t" if CURRENT.free_threaded else ""
+    spec_str = (
+        f"{CURRENT.implementation}{CURRENT.version_info.major}{threaded}-{CURRENT.architecture}-{CURRENT.machine}"
+    )
+    parsed_spec = PythonSpec.from_string_spec(spec_str)
+    assert CURRENT.satisfies(parsed_spec, True) is True
+
+
+def test_py_info_satisfies_not_machine():
+    """Current interpreter should NOT satisfy a spec with a different machine."""
+    other_machine = "arm64" if CURRENT.machine != "arm64" else "x86_64"
+    spec_str = f"{CURRENT.implementation}-{CURRENT.architecture}-{other_machine}"
+    parsed_spec = PythonSpec.from_string_spec(spec_str)
+    assert CURRENT.satisfies(parsed_spec, True) is False
+
+
+def test_py_info_satisfies_no_machine_in_spec():
+    """A spec without machine should match any interpreter (backward compat)."""
+    threaded = "t" if CURRENT.free_threaded else ""
+    spec_str = f"{CURRENT.implementation}{CURRENT.version_info.major}{threaded}-{CURRENT.architecture}"
+    parsed_spec = PythonSpec.from_string_spec(spec_str)
+    assert parsed_spec.machine is None
+    assert CURRENT.satisfies(parsed_spec, True) is True
+
+
+@pytest.mark.parametrize(
+    ("platform", "spec_machine"),
+    [
+        pytest.param("linux-x86_64", "amd64", id="amd64-matches-x86_64"),
+        pytest.param("macosx-14.0-arm64", "aarch64", id="aarch64-matches-arm64"),
+    ],
+)
+def test_py_info_satisfies_machine_cross_os_normalization(platform, spec_machine):
+    info = copy.deepcopy(CURRENT)
+    info.sysconfig_platform = platform
+    spec = PythonSpec.from_string_spec(f"{info.implementation}-{info.architecture}-{spec_machine}")
+    assert info.satisfies(spec, True) is True
+
+
+def test_py_info_to_dict_includes_sysconfig_platform():
+    """_to_dict should include sysconfig_platform."""
+    data = CURRENT._to_dict()  # noqa: SLF001
+    assert "sysconfig_platform" in data
+    assert data["sysconfig_platform"] == sysconfig.get_platform()
+
+
+def test_py_info_json_round_trip():
+    """sysconfig_platform should survive JSON serialization round-trip."""
+    json_str = CURRENT._to_json()  # noqa: SLF001
+    parsed = json.loads(json_str)
+    assert "sysconfig_platform" in parsed
+    restored = PythonInfo._from_json(json_str)  # noqa: SLF001
+    assert restored.sysconfig_platform == CURRENT.sysconfig_platform
+    assert restored.machine == CURRENT.machine
+
+
+@pytest.mark.parametrize(
+    ("target_platform", "discovered_platforms", "expected_idx"),
+    [
+        pytest.param("linux-x86_64", ["linux-aarch64", "linux-x86_64"], 1, id="x86_64-over-aarch64"),
+        pytest.param("macosx-14.0-arm64", ["macosx-14.0-x86_64", "macosx-14.0-arm64"], 1, id="arm64-over-x86_64"),
+    ],
+)
+def test_select_most_likely_prefers_machine_match(target_platform, discovered_platforms, expected_idx):
+    target = copy.deepcopy(CURRENT)
+    target.sysconfig_platform = target_platform
+    discovered = [copy.deepcopy(CURRENT) for _ in discovered_platforms]
+    for d, plat in zip(discovered, discovered_platforms):
+        d.sysconfig_platform = plat
+    result = PythonInfo._select_most_likely(discovered, target)  # noqa: SLF001
+    assert result.sysconfig_platform == discovered_platforms[expected_idx]
